import typer
from distyctl.commands import cmd_init, cmd_dev, cmd_render

app = typer.Typer()

# register subcommands
app.add_typer(cmd_init.app)
app.add_typer(cmd_dev.app)
app.add_typer(cmd_render.app)

if __name__ == "__main__":
    app()



GH_ORGANIZATION_NAME = "Distyopoly"
FRAMEWORK_REPO = "disty-craft"
TEMPORARY_ZIP_BASENAME = ".disty.craft"
FRAMEWORK_RELATIVE_PATH = ".disty-craft"

LOCK_FILE_RELATIVE_PATH = FRAMEWORK_RELATIVE_PATH + "/.disty-craft.lock"



import typer

app = typer.Typer(help="Start the development server")


@app.command(help="Start development mode with hot reload")
def dev():
    """
    Start the development server.

    This command will run the game with automatic reload when source
    files change.
    """
    # typer.echo(f"Running dev server on port {port}, debug={debug}")
    raise NotImplementedError()



from typing import Tuple
from github import Github, GitReleaseAsset
from distyctl import constants, types
from distyctl.exceptions import BrokenFrameworkVersion


class GithubClient:
    def __init__(self):
        self.github_agent = Github()
        self.organization = self.github_agent.get_organization(
            constants.GH_ORGANIZATION_NAME
        )
        self.framework_repo = self.organization.get_repo(constants.FRAMEWORK_REPO)

    def get_latest_release_version(self) -> types.VersionLiteral:
        return self.list_releases()[0]

    def list_releases(self) -> Tuple[str, ...]:
        return tuple(map(lambda x: x.name, self.framework_repo.get_releases()))

    def get_framework_asset_by_version(
        self, version: types.VersionLiteral
    ) -> GitReleaseAsset.GitReleaseAsset:
        release = self.framework_repo.get_release(version)

        assets = release.get_assets()

        first_asset_page = assets.get_page(0)

        if len(first_asset_page) == 0:
            raise BrokenFrameworkVersion(
                version, reason="No assets are included in the release."
            )

        framework_asset = first_asset_page[0]

        return framework_asset



from pathlib import Path
from distyctl import constants, types
from distyctl.exceptions import LockFileFormatError


class LockService:
    def __init__(self, target_path: Path):
        self.lock_path = target_path / constants.LOCK_FILE_RELATIVE_PATH

    def lock_version(self, version: types.VersionLiteral):
        """Locks the project to the specified framework version."""
        self.lock_path.write_text(version)

    def is_locked(self):
        return self.lock_path.is_file()

    def get_locked_version(self) -> types.VersionLiteral:
        if not self.is_locked():
            raise RuntimeError("Project is not locked but lock status was queried.")

        version = self.lock_path.read_text()

        if version == "":
            raise LockFileFormatError()

        return version

    def verify_version(self, current_version: types.VersionLiteral):
        """Compares current_version with the locked version and raises a warning or error if they differ."""
        if self.get_locked_version() == current_version:
            raise RuntimeError("Version verification failed")



from pathlib import Path
from typing import Annotated, Optional
import typer

from distyctl.services.init_service import InitService
from distyctl.types import VersionLiteral

app = typer.Typer(help="Initialize the project environment and files")


@app.command(help="Start development mode with hot reload")
def init(
    target_path: Annotated[
        Path,
        typer.Argument(
            show_default=False,
            default_factory=Path.cwd,
            exists=True,
            file_okay=False,
            dir_okay=True,
            resolve_path=True,
            writable=True,
        ),
    ],
    version: Annotated[Optional[VersionLiteral], typer.Option()] = None,
):
    """ """
    InitService(target_path).intialize_new_game(version)



from pathlib import Path
import zipfile
from distyctl import constants
from github import GitReleaseAsset


class FrameworkInstaller:
    def __init__(self, target_path: Path):
        self.target_path = target_path
        self.temporary_zip_path = self.target_path / constants.TEMPORARY_ZIP_BASENAME

    def install(self, framework_asset: GitReleaseAsset.GitReleaseAsset):
        self.download_framework_release(framework_asset)
        self.unzip_framework_release()
        self.cleanup()

    def download_framework_release(
        self, framework_asset: GitReleaseAsset.GitReleaseAsset
    ):
        framework_asset.download_asset(str(self.temporary_zip_path))

    def unzip_framework_release(self):
        extract_path = self.target_path / constants.FRAMEWORK_RELATIVE_PATH
        # ensure directory exists
        extract_path.mkdir(parents=True, exist_ok=True)

        with zipfile.ZipFile(
            self.target_path / constants.TEMPORARY_ZIP_BASENAME, "r"
        ) as zip_ref:
            zip_ref.extractall(extract_path)

    def cleanup(self):
        self.temporary_zip_path.unlink()



from typing import Annotated, Literal
import typer


app = typer.Typer(help="Prepare the game for deployment")

ModeLiteral = Literal["platform", "standalone"]


@app.command(
    help="Prepares either a standalone deployable game or a game ready to deploy to the Distyopoly platform."
)
def render(mode: Annotated[ModeLiteral, typer.Argument()] = "standalone"):
    """ """
    # typer.echo(f"Running dev server on port {port}, debug={debug}")
    raise NotImplementedError()



from typing import Optional


class BrokenFrameworkVersion(Exception):
    """Raised when the requested framework version is broken or unusable."""

    def __init__(
        self,
        version: str,
        reason: Optional[str] = None,
        issue_url: Optional[str] = None,
    ):
        self.version = version
        self.reason = reason
        self.issue_url = issue_url
        msg = f"Framework version '{version}' is broken."
        if reason:
            msg += f" Reason: {reason}"
        if issue_url:
            msg += f" See: {issue_url}"
        super().__init__(msg)


class LockFileFormatError(Exception):
    """Raised when the lock file is malformed or unreadable."""

    pass



from pathlib import Path
from typing import Optional
from distyctl.libs.framework_installer import FrameworkInstaller
from distyctl.libs.github_client import GithubClient
from distyctl.services.lock_service import LockService
from distyctl.types import VersionLiteral


class InitService:
    """ """

    def __init__(self, target_path: Path):
        self.lock_service = LockService(target_path)
        self.github_client = GithubClient()
        self.framework_installer = FrameworkInstaller(target_path)
        self.target_path = target_path

    def intialize_new_game(self, initial_version: Optional[VersionLiteral] = None):
        version: str
        if initial_version is None:
            if self.lock_service.is_locked():
                version = self.lock_service.get_locked_version()

            else:
                version = self.github_client.get_latest_release_version()
        else:
            version = initial_version

        framework_asset = self.github_client.get_framework_asset_by_version(version)

        self.framework_installer.install(framework_asset)
        self.lock_service.lock_version(version)
        # TODO: add .disty-craft to .gitignore
        # TODO: setup tooling
